#include <stdio.h>
#include <stdlib.h>
#include "ebuild.h"

#define MAXLENGTH 66000
#define NBLOCKS 4
#define INTSIZE 4

/* 
 * the following definition has been taken from
 * /afs/cern.ch/project/cvs/reps/lhcb/DAQ/MDF/MDF/MDFHeader.h
 * CVS revision v1.2
 */
struct MDFHeader  {
    /// Data member indicating the size of the event
    unsigned int   m_size[3];
    /// Optional checksum over the event data (if 0, no checksum was calculated)
    unsigned int   m_checkSum;
    /// Identifier of the compression algorithm used to compress the data buffer
    unsigned char  m_compression;
    /// Event type identifier
    unsigned char  m_evType;
    /// Header type: split into { version:4, length:4 } for possible future upgrade
    unsigned char  m_hdr;
    /// High part of the 40 bit L0 trigger number
    unsigned char  m_trH;
    /// Low part of the 40 bit L0 trigger number
    unsigned int   m_trL;
    /// Trigger mask used for event selection
    unsigned int   m_trMask[4];
};


int main (int argc, char *argv[]) {

  // Reads non-zero suppressed data from the TELL1 and writes the events
  // to a binary file for later decoding by for instance Gaudi/onLine
  // software. Takes the raw bank data and concatinates several event 
  // with an event header between. The header is a 32-bit word
  // containing the lenght of the event, not including the header itself.

  // Takes two arguments, file name and the number of events to be
  // read before the file is closed and the program exits.

  // Declare variables
  int8_t buffer[MAXLENGTH];
  int32_t buffer_size, event_size;
/*   int32_t bank_head_0 = 0; */
/*   int32_t bank_head_1 = 0; */
  int32_t evtNo, nEvents;
  FILE *outfile;
  struct MDFHeader mdf;

  // Check that int is really four bytes!
  if(sizeof(buffer_size)!=INTSIZE) {
    printf("ERROR: type interger not %d bytes as expected\n", INTSIZE);
    abort();
  }

  // Open output file
  if(argc==3) {
    outfile = fopen(argv[1], "wb");
    nEvents = atoi(argv[2]);
  }  else { 
    printf("Expects two arguments:\n");
    printf("writeEventToBinary <filename> <number of events>\n");
    abort();
  }
  if(outfile==NULL) {perror("fopen"); abort();}

  // Create fake bank header (since TELL1 doesn't currently send bank headers.)
/*   bank_head_0 += 0xCBCB;     // Magic pattern */
/*   bank_head_1 += 1;          // Version (first version of data format) */
/*   bank_head_1 += (18<<8);    // Class (velo full data) */
/*   bank_head_1 += (13<<16);   // Source ID (TELL1 number = 13 just for fun) */
  
  printf ("Initialize ebuild library...\n");
  ebuild_init ();

  printf("Expects %d events to write to file %s\n", nEvents, argv[1]);
  
  evtNo=0;
  while(evtNo<nEvents) {

    buffer_size = ebuild_get_event (buffer, MAXLENGTH);
    
    // Write the MDF header
    event_size = buffer_size; // + 8;
    //  fwrite(&event_size, 1, 4, outfile);

    mdf.m_size[0] = mdf.m_size[1] = mdf.m_size[2] = event_size;
    mdf.m_compression = mdf.m_checkSum = 0; /* means we have NO checksum  nor
					     compression */
    mdf.m_evType = 1; /* Markus said so */
    mdf.m_hdr = 0;
    mdf.m_trH = 0;
    mdf.m_trL = ++evtNo;
    mdf.m_trMask[0] = 0x103;
    mdf.m_trMask[1] = mdf.m_trMask[2] = mdf.m_trMask[3] = 0;
    fwrite(&mdf, sizeof(struct MDFHeader), 1, outfile);
//    printf("%08x ", event_size); 
/*     // Write bank header */
/*     bank_head_0 &= 0x0000FFFF;              // Reset event size */
/*     bank_head_0 += ((event_size/4) << 16);  // Set event size (in 32-bit words) */
/*     fwrite(&bank_head_0, INTSIZE, 1, outfile); */
/*     fwrite(&bank_head_1, INTSIZE, 1, outfile); */
    
    // Write bank body
    fwrite(buffer, buffer_size, 1, outfile); 
    //printf("%08x\n", ((u_int16_t *) buffer)[1]);
  }


  // Close file and exit
  if(fclose(outfile)==0) {
    printf("File closed correctly\n");
  } else {
    printf("Error closing file: aborting\n");
    abort();
  }
  return 0;
}
